# モーガンフィンガープリントを用いdnn_regressionで水溶解度を推測
# 参考 https://qiita.com/maskot1977/items/2e98d76a0782a6e89bde
# データはhttp://modem.uscd.edu/adme/data/databases/logS/data_set.datより
# google colab　で動作確認　2022/2/28
# copyright M. Kogoshi & S. Kogoshi

# rdkitのインストール開始(pip使用) 1回目の計算のみ必要 2回目以降は#を付ける
!pip install rdkit
#condaを使う場合は上の行をコメントにし、以下のコメント記号を除いて下さい
"""
!wget -c https://repo.continuum.io/miniconda/Miniconda3-latest-Linux-x86_64.sh
!chmod +x Miniconda3-latest-Linux-x86_64.sh
!bash ./Miniconda3-latest-Linux-x86_64.sh -b -f -p /usr/local
!conda install -q -y -c rdkit rdkit python=3.7
import sys
sys.path.append('/usr/local/lib/python3.7/site-packages/')
"""
# rdkitのインストール終了


from tensorflow import keras
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Dense, Activation, Dropout
import numpy as np
import matplotlib.pyplot as plt
import sys
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import mean_squared_error
from sklearn.preprocessing import StandardScaler
from tensorflow.keras.utils import to_categorical
%matplotlib inline

# データの読み込み
df1 = pd.read_csv('water_solubility_train.txt', sep='\t', header=None)


from rdkit import Chem, DataStructs

#Chem.MolFromSmilesはsmile形式からRDkit用のMolオブジェクトに変換
mols = []
for smile in df1[0]:
    mols.append(Chem.MolFromSmiles(smile))


from rdkit.Chem import AllChem

# morganfingerprintの計算

import numpy as np
fingerprints = []
safe = []

for mol_idx, mol in enumerate(mols):
    try:
#Molオブジェクトをmorganfingerprintに変換してfingerprintに格納
        fingerprint = AllChem.GetMorganFingerprintAsBitVect(mol, 2, 3072)
        fingerprints.append(fingerprint)
        safe.append(mol_idx)
    except:
        print("Error", mol_idx)
        continue

# fingerprintsをnp.array形式に変換
fingerprints = np.array(fingerprints)

print(fingerprints.shape)

# panda形式に変換
pd.DataFrame(fingerprints).head()

x_train = fingerprints

y = df1.iloc[:,2]


# input and output for keras
x_train = x_train.astype(np.float)

X = x_train

# number,dimensio,class_number
(n_samples, n_features) = X.shape


model = Sequential()

# 5 layers
# 1st　layer
model.add(Dense(units=128, activation='relu', input_shape=(n_features,)))
# Dropout
model.add(Dropout(0.1))
# Regularizer　dropoutの代替用　過学習防止
#model.add(Dense(64, input_dim=128,
#                kernel_regularizer=regularizers.l2(0.01),
#                activity_regularizer=regularizers.l2(0.01)))
# second layer
model.add(Dense(units=64, activation='relu'))
# Dropout
model.add(Dropout(0.1))
# 3rd layer
model.add(Dense(units=32, activation='relu'))
# Dropout
model.add(Dropout(0.1))
# 4th layer
#model.add(Dense(units=32, activation='relu'))
# Dropout
#model.add(Dropout(0.1))
# output layer
model.add(Dense(1))

# compile
model.compile(loss='mean_squared_error', optimizer='adam', metrics=['mae'])

# learning
history = model.fit(X, y, epochs=300, validation_split=0.1, batch_size=16, verbose=2)

#loss_graph
plt.xlabel('time step')
plt.ylabel('loss val_loss')
plt.ylim(0,5)
loss, = plt.plot(history.history['loss'], c='#E69F00')
val_loss, = plt.plot(history.history['val_loss'], c='#56B4E9')
plt.show()



# ここから推測
#load of new_data for test

new_data = pd.read_csv('water_solubility_test.txt', sep='\t', header=None)

#Chem.MolFromSmilesはsmile形式からRDkit用のMolオブジェクトに変換
mols_test = []
for smile in new_data[0]:
    mols_test.append(Chem.MolFromSmiles(smile))


from rdkit.Chem import AllChem

# morganfingerprintの計算

fingerprints_test = []
safe_test = []

for mol_idx, mol in enumerate(mols_test):
    try:
#Molオブジェクトをmorganfingerprintに変換してfingerprintに格納
#        fingerprint = [x for x in AllChem.GetMorganFingerprintAsBitVect(mol, 2, 2048)]
        fingerprint = AllChem.GetMorganFingerprintAsBitVect(mol, 2, 3072)
        fingerprints_test.append(fingerprint)
        safe_test.append(mol_idx)
    except:
        print("Error", mol_idx)
        continue

# fingerprintsをnp.array形式に変換
fingerprints_test = np.array(fingerprints_test)


print(fingerprints_test.shape)
#(8991, 2048)

# panda形式に変換
pd.DataFrame(fingerprints_test).head()
x_test = fingerprints_test


# input for keras
x_test = x_test.astype(np.float)


#y=pd.DataFrame(new_data['class_20'])
y_test = new_data.iloc[:,2]

# result
y_pred = model.predict(x_test, verbose=0)

mse = mean_squared_error(y_test,y_pred)

# csv output of test result
y_test = np.array(y_test)
data1 = np.c_[y_test, y_pred]
data1 = pd.DataFrame(data1)
np.savetxt('out.csv', data1, delimiter=',')


plt.xlabel('target_logS')
plt.ylabel('predicted_logS')
plt.xlim(-10,2)
plt.ylim(-10,2)

plt.scatter(y_test,y_pred,s=5)
plt.show()

print("KERAS RMSE: %.2f" %(mse**0.5))


